@@ -330,8 +330,15 @@ function writeStatusFile(dir, state) {
  * Removes the Updates Directory
  */
 function cleanUpUpdatesDir() {
-  var updateDir = getUpdatesDir();
-  
+  // Bail out if we don't have appropriate permissions
+  var updateDir;
+  try {
+    updateDir = getUpdatesDir();
+  }
+  catch (e) {
+    return;
+  }
+        
   var e = updateDir.directoryEntries;
   while (e.hasMoreElements()) {
     var f = e.getNext().QueryInterface(Components.interfaces.nsIFile);
@@ -504,6 +511,7 @@ function readStringFromFile(file) {
  * @constructor
  */
 function UpdatePatch(patch) {
+  this._properties = {};
   for (var i = 0; i < patch.attributes.length; ++i) {
     var attr = patch.attributes.item(i);
     attr.QueryInterface(Components.interfaces.nsIDOMAttr);
@@ -542,7 +550,7 @@ UpdatePatch.prototype = {
   /**
    * A hash of custom properties
    */
-  _properties: { },
+  _properties: null,
   
   /**
    * See nsIWritablePropertyBag.idl
@@ -624,6 +632,7 @@ UpdatePatch.prototype = {
  * @constructor
  */
 function Update(update) {
+  this._properties = {};
   this._patches = [];
   this.installDate = 0;
   this.isCompleteUpdate = false;
@@ -775,7 +784,7 @@ Update.prototype = {
   /**
    * A hash of custom properties
    */
-  _properties: { },
+  _properties: null,
   
   /**
    * See nsIWritablePropertyBag.idl
@@ -922,7 +931,17 @@ UpdateService.prototype = {
    */
   _postUpdateProcessing: function() {
     // Detect installation failures and notify
+    
+    // Bail out if we don't have appropriate permissions
+    if (!this.canUpdate)
+      return;
+      
     var status = readStatusFile(getUpdatesDir()); 
+
+    // Make sure to cleanup after an update that failed for an unknown reason
+    if (status == "null")
+      status = null;
+
     if (status == STATE_DOWNLOADING) {
       LOG("UpdateService", "_postUpdateProcessing: Downloading patch, resuming...");
     }
@@ -950,6 +969,18 @@ UpdateService.prototype = {
       var bundle = sbs.createBundle(URI_UPDATES_PROPERTIES);
       if (status == STATE_SUCCEEDED) {
         update.statusText = bundle.GetStringFromName("installSuccess");
+        
+        // Dig through the update history to find the patch that was just
+        // installed and update its metadata.
+        for (var i = 0; i < um.updateCount; ++i) {
+          var umUpdate = um.getUpdateAt(i);
+          if (umUpdate && umUpdate.version == update.version &&
+                          umUpdate.buildID == update.buildID) {
+            umUpdate.statusText = update.statusText;
+            break;
+          }
+        }
+
         LOG("UpdateService", "_postUpdateProcessing: Install Succeeded, Showing UI");
         prompter.showUpdateInstalled(update);
 
@@ -1376,14 +1407,9 @@ UpdateManager.prototype = {
                                .createInstance(Components.interfaces.nsIFileInputStream);
     fileStream.init(file, MODE_RDONLY, PERMS_FILE, 0);
     try {
-      var sis =
-          Components.classes["@mozilla.org/scriptableinputstream;1"].
-          createInstance(Components.interfaces.nsIScriptableInputStream);
-      sis.init(fileStream);
-      var data = sis.read(sis.available());
       var parser = Components.classes["@mozilla.org/xmlextras/domparser;1"]
                             .createInstance(Components.interfaces.nsIDOMParser);
-      var doc = parser.parseFromString(data, "text/xml");
+      var doc = parser.parseFromStream(fileStream, "UTF-8", fileStream.available(), "text/xml");
       
       var updateCount = doc.documentElement.childNodes.length;
       for (var i = 0; i < updateCount; ++i) {
@@ -1408,9 +1434,15 @@ UpdateManager.prototype = {
    * Load the update manager, initializing state from state files.
    */
   _ensureUpdates: function() {
-    if (!this._updates)
+    if (!this._updates) {
       this._updates = this._loadXMLFileIntoArray(getFile(KEY_APPDIR, 
                         [FILE_UPDATES_DB]));
+
+      // Make sure that any active update is part of our updates list
+      var active = this.activeUpdate;
+      if (active)
+        this._addUpdate(active);
+    }
   },
 
   /**
@@ -2434,7 +2466,7 @@ TimerManager.prototype = {
   }
 };
 
-//@line 2438 "/builds/tinderbox/Tb-Mozilla1.8/Linux_2.4.18-14_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
+//@line 2470 "/builds/tinderbox/Tb-Mozilla1.8/Linux_2.4.18-14_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
 /**
  * UpdatePrompt
  * An object which can prompt the user with information about updates, request
@@ -2576,7 +2608,7 @@ UpdatePrompt.prototype = {
     return this;
   }
 };
-//@line 2580 "/builds/tinderbox/Tb-Mozilla1.8/Linux_2.4.18-14_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
+//@line 2612 "/builds/tinderbox/Tb-Mozilla1.8/Linux_2.4.18-14_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
 
 var gModule = {
   registerSelf: function(componentManager, fileSpec, location, type) {
@@ -2628,13 +2660,13 @@ var gModule = {
                className  : "Update Checker",
                factory    : #1#(Checker)
              },
-//@line 2632 "/builds/tinderbox/Tb-Mozilla1.8/Linux_2.4.18-14_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
+//@line 2664 "/builds/tinderbox/Tb-Mozilla1.8/Linux_2.4.18-14_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
     prompt:  { CID        : Components.ID("{27ABA825-35B5-4018-9FDD-F99250A0E722}"),
                contractID : "@mozilla.org/updates/update-prompt;1",
                className  : "Update Prompt",
                factory    : #1#(UpdatePrompt)
              },
-//@line 2638 "/builds/tinderbox/Tb-Mozilla1.8/Linux_2.4.18-14_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
+//@line 2670 "/builds/tinderbox/Tb-Mozilla1.8/Linux_2.4.18-14_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
     timers:  { CID        : Components.ID("{B322A5C0-A419-484E-96BA-D7182163899F}"),
                contractID : "@mozilla.org/updates/timer-manager;1",
                className  : "Timer Manager",
@@ -2665,14 +2697,14 @@ function NSGetModule(compMgr, fileSpec) {
  *          the specified update, false otherwise.
  */
 function isCompatible(update) {
-//@line 2669 "/builds/tinderbox/Tb-Mozilla1.8/Linux_2.4.18-14_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
+//@line 2701 "/builds/tinderbox/Tb-Mozilla1.8/Linux_2.4.18-14_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
   var em = 
       Components.classes["@mozilla.org/extensions/manager;1"].
       getService(Components.interfaces.nsIExtensionManager);
   var items = em.getIncompatibleItemList("", update.extensionVersion,
     nsIUpdateItem.TYPE_ADDON, false, { });
   return items.length == 0;
-//@line 2678 "/builds/tinderbox/Tb-Mozilla1.8/Linux_2.4.18-14_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
+//@line 2710 "/builds/tinderbox/Tb-Mozilla1.8/Linux_2.4.18-14_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
 }
 
 /**
@@ -2691,7 +2723,7 @@ function showPromptIfNoIncompatibilities(update) {
     prompter.showUpdateAvailable(update);
   }
 
-//@line 2697 "/builds/tinderbox/Tb-Mozilla1.8/Linux_2.4.18-14_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
+//@line 2729 "/builds/tinderbox/Tb-Mozilla1.8/Linux_2.4.18-14_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
   /**
    * Determines if an addon is compatible with a particular update.
    * @param   addon
@@ -2780,17 +2812,17 @@ function showPromptIfNoIncompatibilities(update) {
   };
   
   if (!isCompatible(update)) {
-    var listener = new Listener(em.getIncompatibleItemList("", 
-      update.extensionVersion, nsIUpdateItem.TYPE_ADDON, false, { }));
     var em = 
         Components.classes["@mozilla.org/extensions/manager;1"].
         getService(Components.interfaces.nsIExtensionManager);
+    var listener = new Listener(em.getIncompatibleItemList("", 
+      update.extensionVersion, nsIUpdateItem.TYPE_ADDON, false, { }));
     // See documentation on |mode| above. 
     var mode = getPref("getIntPref", PREF_APP_UPDATE_INCOMPATIBLE_MODE, 
                        INCOMPATIBLE_MODE_NEWVERSIONS);
     em.update([], 0, mode != 0, listener);
   }
   else
-//@line 2797 "/builds/tinderbox/Tb-Mozilla1.8/Linux_2.4.18-14_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
+//@line 2829 "/builds/tinderbox/Tb-Mozilla1.8/Linux_2.4.18-14_Depend/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
     showPrompt(update);
 }
