@@ -1,12 +1,5 @@
-//@line 37 "/builds/tinderbox/thunderbird-trunk/Linux_2.4.18-14_Depend/mozilla/toolkit/mozapps/extensions/content/update.js"
+//@line 37 "/builds/tinderbox/Tb-Mozilla1.8/Linux_2.4.18-14_Depend/mozilla/toolkit/mozapps/extensions/content/update.js"
 
-//
-// window.arguments[1...] is an array of nsIUpdateItem implementing objects 
-// that are to be updated. 
-//  * if the array is empty, all items are updated (like a background update
-//    check)
-//  * if the array contains one or two UpdateItems, with null id fields, 
-//    all items of that /type/ are updated.
 //
 // This UI can be opened from the following places, and in the following modes:
 //
@@ -19,25 +12,12 @@
 //    with the version of the application being started. 
 //    
 //    In this mode, the wizard is opened to panel 'mismatch'. 
-//
-// - from the Extension Manager or Options Dialog or any UI where the user
-//   directly requests to check for updates.
-//    in this case the user selects UpdateItem(s) to update and this list
-//    is passed to this UI. If a single item is sent with the id field set to
-//    null but the type set correctly, this UI will check for updates to all 
-//    items of that type.
-//
-//    In this mode, the wizard is opened to panel 'checking'.
-//
 
-const nsIUpdateItem       = Components.interfaces.nsIUpdateItem;
+const nsIUpdateItem = Components.interfaces.nsIUpdateItem;
+const nsIAUCL = Components.interfaces.nsIAddonUpdateCheckListener;
 
 const PREF_UPDATE_EXTENSIONS_ENABLED            = "extensions.update.enabled";
 const PREF_UPDATE_EXTENSIONS_AUTOUPDATEENABLED  = "extensions.update.autoUpdateEnabled";
-const PREF_UPDATE_EXTENSIONS_COUNT              = "extensions.update.count";
-const PREF_UPDATE_EXTENSIONS_SEVERITY_THRESHOLD = "extensions.update.severity.threshold";
-
-const PREF_UPDATE_SEVERITY                      = "update.severity";
 
 var gShowMismatch = null;
 var gUpdateTypes  = null;
@@ -59,35 +39,19 @@ var gUpdateWizard = {
   
   init: function ()
   {
-    gUpdateTypes = window.arguments[0];
-    gShowMismatch = window.arguments[1];
+    var em = Components.classes["@mozilla.org/extensions/manager;1"]
+                        .getService(Components.interfaces.nsIExtensionManager);
+    // Retrieve all items in order to sync their app compatibility information
+    this.items = em.getItemList(nsIUpdateItem.TYPE_ADDON, { });
+    var pref = 
+        Components.classes["@mozilla.org/preferences-service;1"].
+        getService(Components.interfaces.nsIPrefBranch);
+    this.shouldSuggestAutoChecking = 
+      gShowMismatch && 
+      !pref.getBoolPref(PREF_UPDATE_EXTENSIONS_AUTOUPDATEENABLED);
 
-    var items = window.arguments;
-    if (window.arguments.length == 2) {
-      var em = Components.classes["@mozilla.org/extensions/manager;1"]
-                         .getService(Components.interfaces.nsIExtensionManager);
-      items = [0,0].concat(em.getUpdateableItemList(gUpdateTypes, { }));
-    }
-    
-    for (var i = 2; i < items.length; ++i)
-      this.items.push(items[i].QueryInterface(nsIUpdateItem));
-
-    var pref = Components.classes["@mozilla.org/preferences-service;1"]
-                         .getService(Components.interfaces.nsIPrefBranch);
-    this.shouldSuggestAutoChecking = gShowMismatch && 
-                                     !pref.getBoolPref(PREF_UPDATE_EXTENSIONS_AUTOUPDATEENABLED);
-
-    if (gShowMismatch) 
-      gMismatchPage.init();
-    else
-      document.documentElement.advance();
-  },
-  
-  uninit: function ()
-  {
-    // Ensure all observers are unhooked, just in case something goes wrong or the
-    // user aborts. 
-    gUpdatePage.uninit();  
+    document.documentElement.currentPage = 
+      document.getElementById("versioninfo");
   },
   
   onWizardFinish: function ()
@@ -95,25 +59,7 @@ var gUpdateWizard = {
     var pref = Components.classes["@mozilla.org/preferences-service;1"]
                          .getService(Components.interfaces.nsIPrefBranch);
     if (this.shouldSuggestAutoChecking)
-      pref.setBoolPref(PREF_EXTENSIONS_UPDATE_ENABLED, this.shouldAutoCheck); 
-    
-    if (this.succeeded) {
-      // Downloading and Installed Extension
-      this.clearExtensionUpdatePrefs();
-    }
-
-    // Send an event to refresh any FE notification components. 
-    var os = Components.classes["@mozilla.org/observer-service;1"]
-                       .getService(Components.interfaces.nsIObserverService);
-    os.notifyObservers(null, "Update:Ended", "1");
-  },
-  
-  clearExtensionUpdatePrefs: function ()
-  {
-    var pref = Components.classes["@mozilla.org/preferences-service;1"]
-                         .getService(Components.interfaces.nsIPrefBranch);
-    if (pref.prefHasUserValue(PREF_UPDATE_EXTENSIONS_COUNT)) 
-      pref.clearUserPref(PREF_UPDATE_EXTENSIONS_COUNT);
+      pref.setBoolPref(PREF_UPDATE_EXTENSIONS_ENABLED, this.shouldAutoCheck); 
   },
   
   _setUpButton: function (aButtonID, aButtonKey, aDisabled)
@@ -176,36 +122,99 @@ var gUpdateWizard = {
   }
 };
 
-var gMismatchPage = {
-  init: function ()
+var gVersionInfoPage = {
+  _completeCount: 0,
+  _totalCount: 0,
+  onPageShow: function ()
   {
-    var incompatible = document.getElementById("mismatch.incompatible");
-    for (var i = 0; i < gUpdateWizard.items.length; ++i) {
-      var item = gUpdateWizard.items[i];
-      var listitem = document.createElement("listitem");
-      listitem.setAttribute("label", item.name + " " + item.version);
-      incompatible.appendChild(listitem);
+    gUpdateWizard.setButtonLabels(null, true, 
+                                  "nextButtonText", true, 
+                                  "cancelButtonText", false);
+    var em = Components.classes["@mozilla.org/extensions/manager;1"]
+                       .getService(Components.interfaces.nsIExtensionManager);
+    // Synchronize the app compatibility info for all items by specifying 2 for
+    // the versionUpdateOnly parameter.
+    em.update([], 0, 2, this);
+  },
+
+  /////////////////////////////////////////////////////////////////////////////
+  // nsIAddonUpdateCheckListener
+  onUpdateStarted: function() {
+    this._totalCount = gUpdateWizard.items.length;
+  },
+  
+  onUpdateEnded: function() {
+    var em = Components.classes["@mozilla.org/extensions/manager;1"]
+                       .getService(Components.interfaces.nsIExtensionManager);
+    // Retrieve the remaining incompatible items.
+    gUpdateWizard.items = em.getIncompatibleItemList(null, null,
+                                                     nsIUpdateItem.TYPE_ADDON,
+                                                     true, { });
+    if (gUpdateWizard.items.length > 0) {
+      // There are still incompatible addons, inform the user.
+      document.documentElement.currentPage = 
+        document.getElementById("mismatch");
+    }
+    else {
+      // VersionInfo compatibility updates resolved all compatibility problems,
+      // close this window and continue starting the application...
+      // We need to click the cancel button to wake up the ui due to the way the
+      // EM is using xmlHttpRequest when checking for updates.
+      document.documentElement.getButton("cancel").click();
     }
   },
   
+  onAddonUpdateStarted: function(addon) {
+  },
+  
+  onAddonUpdateEnded: function(addon, status) {
+    if (status == nsIAUCL.STATUS_VERSIONINFO) {
+      for (var i = 0; i < gUpdateWizard.items.length; ++i) {
+        var item = gUpdateWizard.items[i].QueryInterface(nsIUpdateItem);
+        if (addon.id == item.id) {
+          gUpdateWizard.items.splice(i, 1);
+          break;
+        }
+      }
+    }
+    ++this._completeCount;
+
+    // Update the status text and progress bar
+    var progress = document.getElementById("versioninfo.progress");
+    progress.mode = "normal";
+    progress.value = Math.ceil((this._completeCount / this._totalCount) * 100);
+  },
+  
+  /////////////////////////////////////////////////////////////////////////////
+  // nsISupports
+  QueryInterface: function(iid) {
+    if (!iid.equals(Components.interfaces.nsIAddonUpdateCheckListener) && 
+        !iid.equals(Components.interfaces.nsISupports))
+      throw Components.results.NS_ERROR_NO_INTERFACE;
+    return this;
+  }
+};
+
+var gMismatchPage = {
   onPageShow: function ()
   {
     gUpdateWizard.setButtonLabels(null, true, 
                                   "mismatchCheckNow", false, 
                                   "mismatchDontCheck", false);
     document.documentElement.getButton("next").focus();
+
+    var incompatible = document.getElementById("mismatch.incompatible");
+    for (var i = 0; i < gUpdateWizard.items.length; ++i) {
+      var item = gUpdateWizard.items[i].QueryInterface(nsIUpdateItem);
+      var listitem = document.createElement("listitem");
+      listitem.setAttribute("label", item.name + " " + item.version);
+      incompatible.appendChild(listitem);
+    }
   }
 };
 
 var gUpdatePage = {
   _completeCount: 0,
-  _messages: ["Update:Extension:Started", 
-              "Update:Extension:Ended", 
-              "Update:Extension:Item-Started", 
-              "Update:Extension:Item-Ended",
-              "Update:Extension:Item-Error",
-              "Update:Ended"],
-  
   onPageShow: function ()
   {
     gUpdateWizard.setButtonLabels(null, true, 
@@ -213,108 +222,56 @@ var gUpdatePage = {
                                   "cancelButtonText", false);
     document.documentElement.getButton("next").focus();
 
-    var os = Components.classes["@mozilla.org/observer-service;1"]
-                       .getService(Components.interfaces.nsIObserverService);
-    for (var i = 0; i < this._messages.length; ++i)
-      os.addObserver(this, this._messages[i], false);
-
     gUpdateWizard.errorItems = [];
     
     var em = Components.classes["@mozilla.org/extensions/manager;1"]
                        .getService(Components.interfaces.nsIExtensionManager);
-    em.update(gUpdateWizard.items, gUpdateWizard.items.length, false);
+    em.update(gUpdateWizard.items, gUpdateWizard.items.length, false, this);
   },
 
-  _destroyed: false,  
-  uninit: function ()
-  {
-    if (this._destroyed)
-      return;
-  
-    var os = Components.classes["@mozilla.org/observer-service;1"]
-                       .getService(Components.interfaces.nsIObserverService);
-    for (var i = 0; i < this._messages.length; ++i)
-      os.removeObserver(this, this._messages[i]);
-
-    this._destroyed = true;
+  /////////////////////////////////////////////////////////////////////////////
+  // nsIAddonUpdateCheckListener
+  onUpdateStarted: function() {
   },
-
-  _totalCount: 0,
-  get totalCount()
-  {
-    if (!this._totalCount) {
-      this._totalCount = gUpdateWizard.items.length;
-      if (this._totalCount == 0) {
-        var em = Components.classes["@mozilla.org/extensions/manager;1"]
-                           .getService(Components.interfaces.nsIExtensionManager);
-        if (!gUpdateTypes)
-          gUpdateTypes = nsIUpdateItem.TYPE_ADDON;
-
-        this._totalCount = em.getUpdateableItemList(gUpdateTypes, {}).length;
-      }
-    }
-    return this._totalCount;
-  },  
   
-  observe: function (aSubject, aTopic, aData)
-  {
-    var canFinish = false;
-    switch (aTopic) {
-    case "Update:Extension:Started":
-      break;
-    case "Update:Extension:Item-Started":
-      break;
-    case "Update:Extension:Item-Ended":
-      var item = aSubject.QueryInterface(Components.interfaces.nsIUpdateItem);
-      if (aData == "update-check-success")
-        gUpdateWizard.itemsToUpdate.push(item);
-      ++this._completeCount;
-
-      // Update the status text and progress bar
-      var updateStrings = document.getElementById("updateStrings");
-      var status = document.getElementById("checking.status");
-      var statusString = updateStrings.getFormattedString("checkingPrefix", [item.name]);
-      status.setAttribute("value", statusString);
-
-      var progress = document.getElementById("checking.progress");
-      progress.value = Math.ceil((this._completeCount / this.totalCount) * 100);
+  onUpdateEnded: function() {
+    var nextPage = document.getElementById("noupdates");
+    if (gUpdateWizard.itemsToUpdate.length > 0)
+      nextPage = document.getElementById("found");
+    document.documentElement.currentPage = nextPage;
+  },
+  
+  onAddonUpdateStarted: function(addon) {
+  },
+  
+  onAddonUpdateEnded: function(addon, status) {
+    if (status == nsIAUCL.STATUS_UPDATE)
+      gUpdateWizard.itemsToUpdate.push(addon);
+    else if (status == nsIAUCL.STATE_ERROR)
+      gUpdateWizard.errorItems.push(addon);
       
-      break;
-    case "Update:Extension:Item-Error":
-      var item = aSubject.QueryInterface(Components.interfaces.nsIUpdateItem);
-      gUpdateWizard.errorItems.push(item);
-      ++this._completeCount;
+    ++this._completeCount;
 
-      // Update the status text and progress bar
-      var updateStrings = document.getElementById("updateStrings");
-      var status = document.getElementById("checking.status");
-      var statusString = updateStrings.getFormattedString("checkingPrefix", [item.name]);
-      status.setAttribute("value", statusString);
+    // Update the status text and progress bar
+    var updateStrings = document.getElementById("updateStrings");
+    var status = document.getElementById("checking.status");
+    var statusString = updateStrings.getFormattedString("checkingPrefix", [addon.name]);
+    status.setAttribute("value", statusString);
 
-      var progress = document.getElementById("checking.progress");
-      progress.value = Math.ceil((this._completeCount / this.totalCount) * 100);
-
-      break;
-    case "Update:Extension:Ended":
-      canFinish = gUpdateWizard.items.length > 0;
-      break;
-    case "Update:Ended":
-      // If we're doing a general update check, (that is, no specific extensions/themes
-      // were passed in for us to check for updates to), this notification means both
-      // extension and app updates have completed.
-      canFinish = true;
-      break;
-    }
-
-    if (canFinish) {
-      gUpdatePage.uninit();
-      if ((gUpdateTypes & nsIUpdateItem.TYPE_ADDON && gUpdateWizard.itemsToUpdate.length > 0))
-        document.getElementById("checking").setAttribute("next", "found");
-      document.documentElement.advance();
-    }
+    var progress = document.getElementById("checking.progress");
+    progress.value = Math.ceil((this._completeCount / gUpdateWizard.items.length) * 100);
+  },
+  
+  /////////////////////////////////////////////////////////////////////////////
+  // nsISupports
+  QueryInterface: function(iid) {
+    if (!iid.equals(Components.interfaces.nsIAddonUpdateCheckListener) && 
+        !iid.equals(Components.interfaces.nsISupports))
+      throw Components.results.NS_ERROR_NO_INTERFACE;
+    return this;
   }
 };
-
+ 
 var gFoundPage = {
   _nonAppItems: [],
   
@@ -334,6 +291,7 @@ var gFoundPage = {
       checkbox.setAttribute("type", "update");
       checkbox.label        = item.name + " " + item.version;
       checkbox.setAttribute("URL", item.xpiURL);
+      checkbox.setAttribute("hash", item.xpiHash);
       checkbox.infoURL      = "";
       checkbox.internalName = "";
       uri.spec              = item.xpiURL;
@@ -361,11 +319,8 @@ var gFoundPage = {
     var updates = document.getElementById("found.updates");
     if (!this._initialized) {
       this._initialized = true;
-      
       updates.computeSizes();
-
-      if (gUpdateTypes & nsIUpdateItem.TYPE_ADDON)
-        this.buildAddons();
+      this.buildAddons();
     }
         
     var kids = updates._getRadioChildren();
@@ -418,6 +373,7 @@ var gInstallingPage = {
     // Get XPInstallManager and kick off download/install 
     // process, registering us as an observer. 
     var items = [];
+    var hashes = [];
     this._objs = [];
     
     this._restartRequired = false;
@@ -429,13 +385,14 @@ var gInstallingPage = {
     for (var i = 0; i < checkboxes.length; ++i) {
       if (checkboxes[i].type == "update" && checkboxes[i].checked) {
         items.push(checkboxes[i].URL);
+        hashes.push(checkboxes[i].hash ? checkboxes[i].hash : null);
         this._objs.push({ name: checkboxes[i].label });
       }
     }
     
     var xpimgr = Components.classes["@mozilla.org/xpinstall/install-manager;1"]
                            .createInstance(Components.interfaces.nsIXPInstallManager);
-    xpimgr.initManagerFromChrome(items, items.length, this);
+    xpimgr.initManagerWithHashes(items, hashes, items.length, this);
   },
   
   /////////////////////////////////////////////////////////////////////////////
